import 'package:flutter/painting.dart';

class ContrastRatio {
  /// The simplest format, calculates from just one color each
  static double contrastBetweenColors({
    required Color foregroundColor,
    required Color backgroundColor,
    Color solidBackdrop = const Color(0xFFFFFFFF),
  }) {
    final result = contrastRatioBetweenAlphaBlends(
      foregroundAlphaBlends: [foregroundColor],
      backgroundAlphaBlends: [backgroundColor],
      solidBackdrop: solidBackdrop,
    );
    return result;
  }

  /// This is the general base function that will calculate contrast ratio
  /// between any two "situations" by applying subsequent alpha blends
  /// Formulate the blend arrays from bottom to top
  static double contrastRatioBetweenAlphaBlends(
      {required List<Color> foregroundAlphaBlends,
      required List<Color> backgroundAlphaBlends,
      Color solidBackdrop = const Color(0xFFFFFFFF)}) {
    assert(
        foregroundAlphaBlends.isNotEmpty && backgroundAlphaBlends.isNotEmpty);
    Color blendedForegroundColor = foregroundAlphaBlends[0];
    if (foregroundAlphaBlends.length > 1) {
      for (var i = 1; i < foregroundAlphaBlends.length; i++) {
        blendedForegroundColor =
            Color.alphaBlend(blendedForegroundColor, foregroundAlphaBlends[i]);
      }
    }
    Color blendedBackgroundColor = backgroundAlphaBlends[0];
    if (backgroundAlphaBlends.length > 1) {
      for (var i = 1; i < backgroundAlphaBlends.length; i++) {
        blendedBackgroundColor =
            Color.alphaBlend(blendedBackgroundColor, backgroundAlphaBlends[i]);
      }
    }
    final foregroundLuminance = blendedForegroundColor.computeLuminance();
    final backgroundLuminance = blendedBackgroundColor.computeLuminance();
    final greaterLuminance = foregroundLuminance >= backgroundLuminance
        ? foregroundLuminance
        : backgroundLuminance;
    final smallerLuminance = foregroundLuminance >= backgroundLuminance
        ? backgroundLuminance
        : foregroundLuminance;
    final result = (greaterLuminance + 0.05) / (smallerLuminance + 0.05);
    return result;
  }
}
